/********************************************
Copyright Pavel Potapov, 2013-2014
distributed under GNU General Public License
********************************************/

/*************************************
Pavel Potapov, simplest Debug Helper, 2014
**************************************/

/*
While debugging a script one might wish to insert a number of result ("\n"+SOMETHING) 
statements to track the point where the script stops to work. The "Simplest Gebug Helper"
facilitates that. 

Click "buffer". The statement result("\nMARKER ") is now in the memory buffer.
You can past it then at the desired place in the script by CTRL-V. 

It is a good idea to type manually the sequential number in the results statements: 
result("\nMARKER 1") 
result("\nMARKER 2") 
result("\nMARKER ...") 
This way you can localize the script halting point. 

If you wish to monitor the value of the selected variable, type its name in the text 
field. You may add more fields or delete excessive fields by the buttons "add line" and 
"delete line". All the inserted variables will be attached at your result-statement. 

When you finish debugging click "remove all". All the result-statements of 
"result("\nMARKER..." type will be removed from the script.
*/


	
/******************************************************************
DEBUGGER
******************************************************************/
class Debugger112: UIFrame112
{
string Marker1,Marker2

TagGroup Strings
number NumberLines

object StoredRedrawer 
	
TagGroup DLGTags(object self)
	{	
	taggroup Burn = DLGCreateDialog("change").DLGTableLayout(2,2,0)
	Burn.DLGAddElement(DLGCreatePushButton("  buffer  ", "CopyToCash"))
	Burn.DLGAddElement(DLGCreatePushButton("remove all", "RemoveAllMarkers"))

	taggroup Change = DLGCreateDialog("change").DLGTableLayout(2,1,0)
	Change.DLGAddElement(DLGCreatePushButton("add line", "Add"))
	Change.DLGAddElement(DLGCreatePushButton("delete line", "Remove"))

	taggroup Lines = DLGCreateDialog("l")
	for(number i=1;i<=NumberLines;i++)
		{
		string Name 
		try{Strings.TagGroupGetIndexedTagAsString(i-1,Name);} catch{break;}
		Lines.DLGAddElement(DLGCreateStringField(Name,26).DLGIdentifier("string"+i))
		}
		
	taggroup VarList = DLGCreateDialog("").DLGIdentifier("Varbox")	
	VarList.DLGAddElement(Burn)
		VarList.DLGAddElement(Change)
	VarList.DLGAddElement(Lines.DLGanchor("West"))

		
	return VarList
	}
	
void RedrawnUIFrame112(object self) 
	//this methodName is STANDARD and interfaced
	//not change the name, no ARGUMENTS !!
	//This frame drawing can be call from init or from Redrawer
	{
	self.initUIFrame112(self.DLGtags(),self.GetPackageName(),self.GetFrameName())
	}

	
void init(object self,string Package,string Name)
	{
	NumberLines = 1
	Strings = NewTagList()		
 
	Marker1 = "result(\"\\nMARKER" 
	Marker2 = ");"
	
	self.SetPackageName(Package)//needed because UIFrame112 displayed through redrawer
	self.SetFrameName(Name)		//needed because UIFrame112 displayed through redrawer
	
	//FIrst Draw UIFrame
	self.RedrawnUIFrame112()	
	}

void StoreStrings(object self) 
	{
	//store the variables list as global  in case of redrawing
	Strings = NewTagList()
	for(number i=1;i<=NumberLines;i++)
		{	
		Strings.TagGroupInsertTagAsString(i,self.LookUpElement("string"+i).DLGGetStringValue())
		}
	}
	
void Refresh(object self)
	{
	//store basic DLG tags
	//self.SetUIFrameTags(self.DLGTags())

	//lanch redrawer
	alloc(DLGRedrawer112).init(self.ScriptObjectgetID()).StartThread("Go")
	}

//add new variable to track	
void Add(object self) 
	{	
	self.StoreStrings()	
	NumberLines++
	self.Refresh()		
	}

//delete one variable
void Remove(object self) 
	{	
	self.StoreStrings()
	NumberLines--
	self.Refresh()
	}

//replace text fragment in script	
void ChangeFrontMostScriptText(object self,string M1,string M2,string replace)
	{
	//get script text
	documentwindow Win = GetNthDocumentWindowOfType(4,0)
	if(Win.WindowIsValid()&&(Win.WindowGetType()==4)) 
		{

		string Content = Win.EditorWindowGetText()
		
		//replace text metween M1 and M2 for Replace
		Content = ReplaceFragmentInText112(Content,M1,M2,Replace)
		EditorWindowSetText(Win,Content)	
		}
	}

//arrange the marker text and copy it to the PC buffer ready for Paste
void CopyToCash(object self)
	{  
	//build the debug text assiciated with variables
	string ToPrint = Marker1 +" \""
	for(number i=1;i<=NumberLines;i++)
		{
		string var = self.LookUpElement("string"+i).DLGgetStringvalue()
		if(var!="") ToPrint += ("+\" "+var+":\"+"+var)
		}
	ToPrint += Marker2 
//result("\n"+ToPrint)	

	DocumentWindow temp = NewScriptWindow("",200,200,220,220)
	temp.EditorWindowSetText(ToPrint)
	ChooseMenuItem("Edit", "","Select All")
	ChooseMenuItem("Edit", "","Copy")
	temp.WindowClose(0)
	}

//remove all MARKERs from front most script
void RemoveAllMarkers(object self)
	{  
	self.ChangeFrontMostScriptText(Marker1,Marker2,"")	
	}
	
}

void createALL() alloc(Debugger112).init("_temDM-Scripts:Debugger","Debugger")

createAll()